@@ -1,20 +1,20 @@
-from collections import defaultdict
 import logging
-from copy import deepcopy
-
 import random
-from graphic_coloring_engine.errors import ColoringEngineError
+from collections import defaultdict
+from copy import deepcopy
+from dataclasses import dataclass, field
+from typing import Callable, Dict, List, Optional, Set, Union
 
+import pydash as _
 from cached_property import cached_property
+from colormath.color_objects import ColorBase
+from shapely.geometry import MultiPolygon
 from shapely.geometry.polygon import Polygon
+from typing_extensions import Literal
+
+from graphic_coloring_engine.errors import ColoringEngineError
 from graphic_coloring_engine.monkey_patching.colormath_contrast import contrast
 from graphic_coloring_engine.polygon import GeomTypeEnum, multi_polygon_difference
-from typing import Callable, DefaultDict, Dict, List, Optional, Set, Union
-from colormath.color_objects import ColorBase
-from typing_extensions import Literal
-import pydash as _
-from shapely.geometry import MultiPolygon
-from dataclasses import dataclass, field
 
 logger = logging.getLogger(__name__)
 
@@ -67,10 +67,10 @@ class Coordinate:
 
 @dataclass
 class Layer:
-    order: int  # 值越小越底层
+    order: int  # 值越小越顶层
     bbox_coordinate: Coordinate
     type: Literal["image", "text"]
-    polygon: MultiPolygon = field(default=None) # 用于计算碰撞关系
+    polygon: MultiPolygon = field(default=None)  # 用于计算碰撞关系
     dominant_colors: List[DominantColor] = field(default_factory=list)  # 图片才有
     color_mutable: bool = field(default=False)  # 是否需要进行配色
     color: Optional[Union[Color, ColorChoice]] = None
@@ -79,7 +79,7 @@ class Layer:
         if self.type == "image":
             self.color_mutable = False
             if not self.dominant_colors:
-                logger.warn(f'image layer {self.order} has empty dominant colors')
+                logger.warn(f"image layer {self.order} has empty dominant colors")
             if not self.color and self.dominant_color_with_highest_ratio:
                 self.color = self.dominant_color_with_highest_ratio
         if not self.polygon:
@@ -89,7 +89,9 @@ class Layer:
     def dominant_color_with_highest_ratio(self) -> Optional[Color]:
         if not self.dominant_colors:
             return None
-        dominant_color_with_highest_ratio: Union[DominantColor] = _.max_by(self.dominant_colors, lambda c: c.ratio)
+        dominant_color_with_highest_ratio: Union[DominantColor] = _.max_by(
+            self.dominant_colors, lambda c: c.ratio
+        )
         return dominant_color_with_highest_ratio
 
     @property
@@ -104,10 +106,12 @@ class Layout:
     layers: List[Layer]
 
     # layer 碰撞矩阵
-    layer_collision_map: Dict[int, List[int]] = field(default_factory=lambda: defaultdict(list), init=False)
+    layer_collision_map: Dict[int, Set[int]] = field(
+        default_factory=lambda: defaultdict(list), init=False
+    )
 
     def __post_init__(self):
-        self.layers = sorted(self.layers, key=lambda l: l.order)  # 从底往顶排序
+        self.layers = sorted(self.layers, key=lambda l: -l.order)  # 从底往顶排序
         self.layer_collision_map = self._collision_detection()
 
     def __getitem__(self, order: int) -> Optional[Layer]:
@@ -118,7 +122,7 @@ class Layout:
 
     @cached_property
     def layer_map(self) -> Dict[int, Layer]:
-        return _.key_by(self.layers, lambda l: l.order)
+        return {l.order: l for l in self.layers}
 
     def _collision_detection(self) -> Dict[int, Set[int]]:
         """图层碰撞计算"""
@@ -126,7 +130,7 @@ class Layout:
         for _layer in self.layers:
             logger.debug(f"{_layer.order} {_layer.polygon}")
 
-        layer_collision_map = DefaultDict(set)
+        layer_collision_map = defaultdict(set)
         # 按绘制顺序更新每层的 polygon
         logger.debug("开始更新图层的展示区域 polygon")
         for i, layer in enumerate(self.layers):
@@ -152,20 +156,29 @@ class Layout:
                     continue
                 else:
                     # 面相交
-                    intersection = intersection.simplify(SIMPLIFY_RATIO, preserve_topology=False)
+                    intersection = intersection.simplify(
+                        SIMPLIFY_RATIO, preserve_topology=False
+                    )
                     # 更新 prev_layer 的 polygon
-                    prev_layer.polygon = multi_polygon_difference(prev_layer.polygon, intersection)
+                    prev_layer.polygon = multi_polygon_difference(
+                        prev_layer.polygon, intersection
+                    )
                     logger.debug(f"\t\t-> {prev_layer.polygon}")
                     # 更新碰撞矩阵
                     layer_collision_map[layer.order].add(prev_layer.order)
                     layer_collision_map[prev_layer.order].add(layer.order)
+
         logger.debug(f"初步碰撞计算结果")
         for _layer in self.layers:
             logger.debug(f"{_layer.order} {_layer.polygon}")
         # 不显示在画布上的元素
         logger.debug("开始清理画布上不出现的元素")
         layer_orders_not_visible_on_canvas = set(
-            [order for order in layer_collision_map if not self.layer_map[order].visible_on_canvas]
+            [
+                order
+                for order in layer_collision_map
+                if not self.layer_map[order].visible_on_canvas
+            ]
         )
         logger.debug(f"清理元素: {layer_orders_not_visible_on_canvas}")
         # 去除
@@ -211,15 +224,17 @@ class ColoringEngine:
     # 除了画布上的颜色以外还可以用哪些颜色
     extra_usable_colors: Set[ColorChoice] = field(default_factory=set)
     # 用于每个 layer 可用颜色过滤
-    layer_color_filter_map: Dict[int, List[Callable[[ColorChoice, Layout], bool]]] = field(
-        default_factory=lambda: defaultdict(list)
-    )
+    layer_color_filter_map: Dict[
+        int, List[Callable[[ColorChoice, Layout], bool]]
+    ] = field(default_factory=lambda: defaultdict(list))
     # 用于每个 layer 配色过程的校验
-    layer_color_constraint_map: Dict[int, List[Callable[[Color, Layout], bool]]] = field(
-        default_factory=lambda: defaultdict(list)
-    )
+    layer_color_constraint_map: Dict[
+        int, List[Callable[[Color, Layout], bool]]
+    ] = field(default_factory=lambda: defaultdict(list))
     # 用于配色后的全局校验
-    global_color_constraint: List[Callable[[Layout], bool]] = field(default_factory=list)
+    global_color_constraint: List[Callable[[Layout], bool]] = field(
+        default_factory=list
+    )
     # usable colors
     usable_colors: Set[ColorChoice] = field(default_factory=set, init=False)
     # 随机数生成器
@@ -250,8 +265,14 @@ class ColoringEngine:
         return None
 
     @staticmethod
-    def extract_color_scheme(colorized_layout: Layout) -> Dict[int, Union[Color, ColorChoice]]:
-        return {layer.order: layer.color for layer in colorized_layout.layers if layer.color_mutable}
+    def extract_color_scheme(
+        colorized_layout: Layout,
+    ) -> Dict[int, Union[Color, ColorChoice]]:
+        return {
+            layer.order: layer.color
+            for layer in colorized_layout.layers
+            if layer.color_mutable
+        }
 
     def _colorize(self, layout: Optional[Layout] = None) -> Layout:
         """生成器"""
@@ -279,23 +300,25 @@ class ColoringEngine:
         layer_logger.info(f"layer_usable_colors: filtered {self.usable_colors}")
 
         # 验证函数
-        def color_validation_fn(color_choice: ColorChoice) -> bool:
+        def color_validation_fn(_color_choice: ColorChoice) -> bool:
             # 来自外部的约束
-            color_validation_fn = self.get_layer_color_constraint(next_layer.order)
-            if color_validation_fn:
+            _color_validation_fn = self.get_layer_color_constraint(next_layer.order)
+            if _color_validation_fn:
                 try:
-                    color_validation_fn(color_choice)
+                    _color_validation_fn(_color_choice)
                 except ColoringEngineError as e:
                     logger.info(e)
                     return False
             # 来自 collision 的约束
             return all(
                 (
-                    layer.color.contrast(color_choice) > self.constants.文字与背景的最小对比度
+                    layer.color.contrast(_color_choice) > self.constants.文字与背景的最小对比度
                     for layer in _.filter_(
                         [
                             layout.layer_map[collision_layer_order]
-                            for collision_layer_order in layout.layer_collision_map[next_layer.order]
+                            for collision_layer_order in layout.layer_collision_map[
+                                next_layer.order
+                            ]
                         ],
                         lambda layer: layer.color,
                     )
@@ -311,7 +334,12 @@ class ColoringEngine:
             if not self.find_next_layer_to_colorize(layout=layout):
                 should_yield = True
                 if should_yield and self.global_color_constraint:
-                    should_yield = all([constraint_fn(layout) for constraint_fn in self.global_color_constraint])
+                    should_yield = all(
+                        [
+                            constraint_fn(layout)
+                            for constraint_fn in self.global_color_constraint
+                        ]
+                    )
                     if not should_yield:
                         logger.info(f"全局校验失败, {layout}")
                 if should_yield:
@@ -336,7 +364,9 @@ class ColoringEngine:
                 break
         return color_schemes
 
-    def get_layer_color_filters(self, order: int) -> Optional[Callable[[ColorChoice], bool]]:
+    def get_layer_color_filters(
+        self, order: int
+    ) -> Optional[Callable[[ColorChoice], bool]]:
         filters = self.layer_color_filter_map.get(order)
         if not filters:
             return
@@ -344,11 +374,15 @@ class ColoringEngine:
         def filter_color(color_choice: ColorChoice) -> bool:
             for filter_fn in filters:
                 if not filter_fn(color_choice, self):
-                    raise ColoringEngineError(f"业务校验不通过: 颜色过滤, layer-{order}, {color_choice}")
+                    raise ColoringEngineError(
+                        f"业务校验不通过: 颜色过滤, layer-{order}, {color_choice}"
+                    )
 
         return filter_color
 
-    def get_layer_color_constraint(self, order: int) -> Optional[Callable[[ColorChoice], bool]]:
+    def get_layer_color_constraint(
+        self, order: int
+    ) -> Optional[Callable[[ColorChoice], bool]]:
         filters = self.layer_color_constraint_map.get(order)
         if not filters:
             return
@@ -356,7 +390,9 @@ class ColoringEngine:
         def validate_color(color_choice: ColorChoice) -> bool:
             for filter_fn in filters:
                 if not filter_fn(color_choice, self):
-                    raise ColoringEngineError(f"业务校验不通过: 颜色规则, layer-{order}, {color_choice}")
+                    raise ColoringEngineError(
+                        f"业务校验不通过: 颜色规则, layer-{order}, {color_choice}"
+                    )
 
         return validate_color
 
@@ -368,7 +404,9 @@ class ColoringEngine:
 
     def _validate_colorized(self):
         """是否已全部配色"""
-        uncolorized_layers = [layer for layer in self.layout.layers if layer.color_mutable and layer.color]
+        uncolorized_layers = [
+            layer for layer in self.layout.layers if layer.color_mutable and layer.color
+        ]
         if uncolorized_layers:
             raise ColoringEngineError(f"以下节点尚未配色: {uncolorized_layers}")
 
@@ -384,8 +422,13 @@ class ColoringEngine:
                     continue
                 if "text" in set([src_layer.type, collision_layer.type]):
                     # 至少一层文字, 需要满足文字的对比度要求
-                    if src_layer.color.contrast(collision_layer.color) < self.constants.文字与背景的最小对比度:
-                        raise ColoringEngineError(f"文字与背景对比度不满足: {src_layer}, {collision_layer}")
+                    if (
+                        src_layer.color.contrast(collision_layer.color)
+                        < self.constants.文字与背景的最小对比度
+                    ):
+                        raise ColoringEngineError(
+                            f"文字与背景对比度不满足: {src_layer}, {collision_layer}"
+                        )
 
     def _validate_color_constraint(self):
         """业务校验"""
